@@ -42,13 +42,13 @@ from AnyQt.QtCore import (
 )
 from AnyQt.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
 
-from orangecanvas.utils import unique, name_lookup, markup
-from orangecanvas.utils.shtools import python_process, create_process, \
-    temp_named_file
+import sip
 
+from orangecanvas.utils import unique, name_lookup, markup, qualified_name
+from orangecanvas.utils.shtools import python_process, create_process
+from ..utils.pkgmeta import get_dist_meta, parse_meta
 from ..utils.qinvoke import qinvoke
 from ..gui.utils import message_warning, message_critical as message_error
-from ..help.manager import get_dist_meta, parse_meta
 
 from .. import config
 from ..config import Config
@@ -66,6 +66,15 @@ def normalize_name(name):
     return re.sub(r"[-_.]+", "-", name).lower()
 
 
+def prettify_name(name):
+    dash_split = name.split('-')
+    # Orange3-ImageAnalytics => ImageAnalytics
+    if len(dash_split) > 1 and dash_split[0].lower() in ['orange', 'orange3']:
+        name = '-'.join(dash_split[1:])
+    # ImageAnalytics => Image Analytics  # while keeping acronyms
+    return re.sub(r"(?<!^)([A-Z][a-z]|(?<=[a-z])[A-Z])", r" \1", name)
+
+
 class Installable(
     NamedTuple(
         "Installable", (
@@ -75,7 +84,9 @@ class Installable(
             ("description", str),
             ("package_url", str),
             ("release_urls", List['ReleaseUrl']),
+            ("requirements", List[str]),
             ("description_content_type", Optional[str]),
+            ("force", bool),
         ))):
     """
     An installable distribution from PyPi
@@ -95,7 +106,9 @@ class Installable(
     """
 
 Installable.__new__.__defaults__ = (
+    [],
     None,  # description_content_type = None,
+    False,  # force = False
 )
 
 
@@ -195,6 +208,9 @@ def is_updatable(item):
         except ValueError:
             return False
 
+        if inst.force:
+            return True
+
         if item.constraint is not None and str(v2) not in item.constraint:
             return False
         else:
@@ -325,15 +341,15 @@ class PluginsModel(QStandardItemModel):
         if isinstance(item, Installed):
             installed = True
             ins, dist = item.installable, item.local
-            name = dist.project_name
+            name = prettify_name(dist.project_name)
             summary = get_dist_meta(dist).get("Summary", "")
-            version = ins.version if ins is not None else dist.version
+            version = dist.version
             item_is_core = item.required
         else:
             installed = False
             ins = item.installable
             dist = None
-            name = ins.name
+            name = prettify_name(ins.name)
             summary = ins.summary
             version = ins.version
             item_is_core = False
@@ -363,7 +379,8 @@ class PluginsModel(QStandardItemModel):
         if updatable:
             assert dist is not None
             assert ins is not None
-            version = "{} < {}".format(dist.version, ins.version)
+            comp = "<" if not ins.force else "->"
+            version = "{} {} {}".format(dist.version, comp, ins.version)
 
         item3 = QStandardItem(version)
         item3.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable)
@@ -841,9 +858,6 @@ class AddonManagerDialog(QDialog):
         installable: Installable
         """
         items = self.items()
-        if installable.name in {item.installable.name for item in items
-                                if item.installable is not None}:
-            return
         installed = [ep.dist for ep in self.config().addon_entry_points()]
         new_ = installable_items([installable], filter(None, installed))
 
@@ -859,7 +873,13 @@ class AddonManagerDialog(QDialog):
         new = next(filter(match, new_), None)
         assert new is not None
         state = self.itemState()
-        self.setItems(items + [new])
+        replace = next(filter(match, items), None)
+        if replace is not None:
+            items[items.index(replace)] = new
+            self.setItems(items)
+            # the state for the replaced item will be removed by setItemState
+        else:
+            self.setItems(items + [new])
         self.setItemState(state)  # restore state
 
     def addItems(self, items: List[Item]):
@@ -957,13 +977,13 @@ class AddonManagerDialog(QDialog):
         super().done(retcode)
         if self.__thread is not None:
             self.__thread.quit()
-            self.__thread.wait(1000)
+            self.__thread = None
 
     def closeEvent(self, event):
         super().closeEvent(event)
         if self.__thread is not None:
             self.__thread.quit()
-            self.__thread.wait(1000)
+            self.__thread = None
 
     ADDON_EXTENSIONS = ('.zip', '.whl', '.tar.gz')
 
@@ -994,10 +1014,12 @@ class AddonManagerDialog(QDialog):
                 summary = meta.get("Summary", "")
                 descr = meta.get("Description", "")
                 content_type = meta.get("Description-Content-Type", None)
+                requirements = meta.get("Requires-Dist", "")
                 names.append(name)
                 packages.append(
                     Installable(name, vers, summary,
-                                descr or summary, path, [path], content_type)
+                                descr or summary, path, [path], requirements,
+                                content_type, True)
                 )
 
         for installable in packages:
@@ -1014,18 +1036,52 @@ class AddonManagerDialog(QDialog):
     def __accepted(self):
         steps = self.itemState()
 
+        # warn about implicit upgrades of required core packages
+        core_required = {}
+        for item in self.items():
+            if isinstance(item, Installed) and item.required:
+                core_required[item.local.project_name] = item.local.version
+
+        core_upgrade = set()
+        for step in steps:
+            if step[0] in [Upgrade, Install]:
+                inst = step[1].installable
+                if inst.name in core_required:  # direct upgrade of a core package
+                    core_upgrade.add(inst.name)
+                if inst.requirements:  # indirect upgrade of a core package as a requirement
+                    for req in pkg_resources.parse_requirements(inst.requirements):
+                        if req.name in core_required and core_required[req.name] not in req:
+                            core_upgrade.add(req.name)  # current doesn't meet requirements
+
+        if core_upgrade:
+            icon = QMessageBox.Warning
+            buttons = QMessageBox.Ok | QMessageBox.Cancel
+            title = "Warning"
+            text = "This action will upgrade some core packages:\n"
+            text += "\n".join(sorted(core_upgrade))
+            msg_box = QMessageBox(icon, title, text, buttons, self)
+            msg_box.setInformativeText("Do you want to continue?")
+            msg_box.setDefaultButton(QMessageBox.Ok)
+            if msg_box.exec() != QMessageBox.Ok:
+                steps = []
+
         if steps:
             # Move all uninstall steps to the front
             steps = sorted(
                 steps, key=lambda step: 0 if step[0] == Uninstall else 1
             )
             self.__installer = Installer(steps=steps)
-            self.__thread = QThread(self)
-            self.__thread.start()
-
+            self.__thread = QThread(
+                objectName=qualified_name(type(self)) + "::InstallerThread",
+            )
+            # transfer ownership to c++; the instance is (deferred) deleted
+            # from the finished signal (keep alive until then).
+            sip.transferto(self.__thread, None)
+            self.__thread.finished.connect(self.__thread.deleteLater)
             self.__installer.moveToThread(self.__thread)
             self.__installer.finished.connect(self.__on_installer_finished)
             self.__installer.error.connect(self.__on_installer_error)
+            self.__thread.start()
 
             progress = self.progressDialog()
 
@@ -1037,10 +1093,17 @@ class AddonManagerDialog(QDialog):
         else:
             self.accept()
 
-    def __on_installer_error(self, command, pkg, retcode, output):
+    def __on_installer_finished_common(self):
         if self.__progress is not None:
             self.__progress.close()
             self.__progress = None
+
+        if self.__thread is not None:
+            self.__thread.quit()
+            self.__thread = None
+
+    def __on_installer_error(self, command, pkg, retcode, output):
+        self.__on_installer_finished_common()
         message_error(
             "An error occurred while running a subprocess", title="Error",
             informative_text="{} exited with non zero status.".format(command),
@@ -1050,27 +1113,42 @@ class AddonManagerDialog(QDialog):
         self.reject()
 
     def __on_installer_finished(self):
-        if self.__progress is not None:
-            self.__progress.close()
-            self.__progress = None
+        self.__on_installer_finished_common()
+        name = QApplication.applicationName() or 'Orange'
 
         def message_restart(parent):
             icon = QMessageBox.Information
             buttons = QMessageBox.Ok | QMessageBox.Cancel
             title = 'Information'
-            text = 'Orange needs to be restarted for the changes to take effect.'
-
+            text = ('{} needs to be restarted for the changes to take effect.'
+                    .format(name))
             msg_box = QMessageBox(icon, title, text, buttons, parent)
             msg_box.setDefaultButton(QMessageBox.Ok)
-            msg_box.setInformativeText('Press OK to close Orange now.')
-
+            msg_box.setInformativeText('Press OK to restart {} now.'
+                                       .format(name))
             msg_box.button(QMessageBox.Cancel).setText('Close later')
             return msg_box.exec_()
 
         if QMessageBox.Ok == message_restart(self):
             self.accept()
-            QTimer.singleShot(0, QApplication.closeAllWindows)
-            QTimer.singleShot(0, QApplication.quit)
+
+            def restart():
+                quit_temp_val = QApplication.quitOnLastWindowClosed()
+                QApplication.setQuitOnLastWindowClosed(False)
+                QApplication.closeAllWindows()
+                windows = QApplication.topLevelWindows()
+                if any(w.isVisible() for w in windows):  # if a window close was cancelled
+                    QApplication.setQuitOnLastWindowClosed(quit_temp_val)
+                    QMessageBox(
+                        text="Restart Cancelled",
+                        informativeText="Changes will be applied on {}'s next restart"
+                                        .format(name),
+                        icon=QMessageBox.Information
+                    ).exec()
+                else:
+                    QApplication.exit(96)
+
+            QTimer.singleShot(0, restart)
         else:
             self.reject()
 
@@ -1136,9 +1214,10 @@ def installable_from_json_response(meta):
                                python_version=r.get("python_version", ""),
                                package_type=r["packagetype"])
                     for r in distributions]
+    requirements = info.get("requires_dist", [])
 
     return Installable(name, version, summary, description, package_url, release_urls,
-                       content_type)
+                       requirements, content_type)
 
 
 def _session(cachedir=None):
@@ -1230,7 +1309,7 @@ def list_available_versions(config, session=None):
         distributions.append(response.json())
 
     packages = []
-    for addon in defaults + distributions:
+    for addon in distributions + defaults:
         try:
             packages.append(installable_from_json_response(addon))
         except (TypeError, KeyError):
@@ -1370,26 +1449,34 @@ class Installer(QObject):
     @Slot()
     def _next(self):
         command, pkg = self.__queue.popleft()
-
         try:
-            if command == Install:
+            if command == Install \
+                    or (command == Upgrade and pkg.installable.force):
                 self.setStatusMessage(
                     "Installing {}".format(pkg.installable.name))
                 if self.conda:
-                    self.conda.install(pkg.installable, raise_on_fail=False)
-                self.pip.install(pkg.installable)
+                    try:
+                        self.conda.install(pkg.installable)
+                    except CommandFailed:
+                        self.pip.install(pkg.installable)
+                else:
+                    self.pip.install(pkg.installable)
             elif command == Upgrade:
                 self.setStatusMessage(
                     "Upgrading {}".format(pkg.installable.name))
                 if self.conda:
-                    self.conda.upgrade(pkg.installable, raise_on_fail=False)
-                self.pip.upgrade(pkg.installable)
+                    try:
+                        self.conda.upgrade(pkg.installable)
+                    except CommandFailed:
+                        self.pip.upgrade(pkg.installable)
+                else:
+                    self.pip.upgrade(pkg.installable)
             elif command == Uninstall:
                 self.setStatusMessage(
                     "Uninstalling {}".format(pkg.local.project_name))
                 if self.conda:
                     try:
-                        self.conda.uninstall(pkg.local, raise_on_fail=True)
+                        self.conda.uninstall(pkg.local)
                     except CommandFailed:
                         self.pip.uninstall(pkg.local)
                 else:
@@ -1415,36 +1502,31 @@ class PipInstaller:
 
     def install(self, pkg):
         # type: (Installable) -> None
-        constraints = config.default.core_packages()
-        constraints = ("\n".join(constraints))
-        with temp_named_file(constraints, suffix=".txt") as cfile:
-            cmd = [
-                "python", "-m", "pip",  "install",
-                      "--constraint", cfile,
-            ] + self.arguments
-
-            if pkg.package_url.startswith(("http://", "https://")):
-                cmd.append(pkg.name)
-            else:
-                # Package url is path to the (local) wheel
-                cmd.append(pkg.package_url)
-
-            run_command(cmd)
+        cmd = ["python", "-m", "pip",  "install"] + self.arguments
+        if pkg.package_url.startswith(("http://", "https://")):
+            version = (
+                "=={}".format(pkg.version) if pkg.version is not None else ""
+            )
+            cmd.append(pkg.name + version)
+        else:
+            # Package url is path to the (local) wheel
+            cmd.append(pkg.package_url)
+        run_command(cmd)
 
     def upgrade(self, package):
-        constraints = config.default.core_packages()
-        constraints = ("\n".join(constraints))
-        with temp_named_file(constraints, suffix=".txt") as cfile:
-            cmd = [
-                "python", "-m", "pip", "install",
-                    "--upgrade", "--upgrade-strategy=only-if-needed",
-                    "--constraint", cfile,
-            ] + self.arguments
-            if package.package_url.startswith(("http://", "https://")):
-                cmd.append(package.name)
-            else:
-                cmd.append(package.package_url)
-            run_command(cmd)
+        cmd = [
+            "python", "-m", "pip", "install",
+                "--upgrade", "--upgrade-strategy=only-if-needed",
+        ] + self.arguments
+        if package.package_url.startswith(("http://", "https://")):
+            version = (
+                "=={}".format(package.version) if package.version is not None
+                else ""
+            )
+            cmd.append(package.name + version)
+        else:
+            cmd.append(package.package_url)
+        run_command(cmd)
 
     def uninstall(self, dist):
         cmd = ["python", "-m", "pip", "uninstall", "--yes", dist.project_name]
@@ -1477,20 +1559,24 @@ class CondaInstaller:
             os.environ["CONDA_DEFAULT_ENV"] = bin
             return conda
 
-    def install(self, pkg, raise_on_fail=False):
+    def install(self, pkg):
+        version = "={}".format(pkg.version) if pkg.version is not None else ""
         cmd = [self.conda, "install", "--yes", "--quiet",
-               self._normalize(pkg.name)]
-        run_command(cmd, raise_on_fail=raise_on_fail)
+               "--satisfied-skip-solve",
+               self._normalize(pkg.name) + version]
+        return run_command(cmd)
 
-    def upgrade(self, pkg, raise_on_fail=False):
-        cmd = [self.conda, "upgrade", "--yes", "--quiet",
-               self._normalize(pkg.name)]
-        run_command(cmd, raise_on_fail=raise_on_fail)
+    def upgrade(self, pkg):
+        version = "={}".format(pkg.version) if pkg.version is not None else ""
+        cmd = [self.conda, "install", "--yes", "--quiet",
+               "--satisfied-skip-solve",
+               self._normalize(pkg.name) + version]
+        return run_command(cmd)
 
-    def uninstall(self, dist, raise_on_fail=False):
+    def uninstall(self, dist):
         cmd = [self.conda, "uninstall", "--yes",
                self._normalize(dist.project_name)]
-        run_command(cmd, raise_on_fail=raise_on_fail)
+        return run_command(cmd)
 
     def _normalize(self, name):
         # Conda 4.3.30 is inconsistent, upgrade command is case sensitive
