@@ -8,13 +8,14 @@ import logging
 import operator
 import io
 import traceback
+from concurrent import futures
 
 from xml.sax.saxutils import escape
 from functools import partial, reduce
 from types import SimpleNamespace
 from typing import (
     Optional, List, Union, Any, cast, Dict, Callable, IO, Sequence, Iterable,
-    Tuple, TypeVar,
+    Tuple, TypeVar, Awaitable,
 )
 
 import pkg_resources
@@ -48,7 +49,7 @@ from AnyQt.QtCore import (
     pyqtProperty as Property, pyqtSignal as Signal
 )
 
-from ..scheme import Scheme, IncompatibleChannelTypeError
+from ..scheme import Scheme, IncompatibleChannelTypeError, SchemeNode
 from ..scheme import readwrite
 from ..scheme.readwrite import UnknownWidgetDefinition
 from ..gui.dropshadow import DropShadowFrame
@@ -74,8 +75,11 @@ from ..registry import WidgetRegistry, WidgetDescription, CategoryDescription
 from ..registry.qt import QtWidgetRegistry
 from ..utils.settings import QSettings_readArray, QSettings_writeArray
 from ..utils.qinvoke import qinvoke
-from ..utils.pickle import Pickler, Unpickler, glob_scratch_swps, swp_name, canvas_scratch_name_memo
-from ..utils import unique, group_by_all
+from ..utils.pickle import Pickler, Unpickler, glob_scratch_swps, swp_name, \
+    canvas_scratch_name_memo, register_loaded_swp
+from ..utils import unique, group_by_all, set_flag
+from ..utils.asyncutils import get_event_loop
+from ..utils.qobjref import qobjref
 
 from . import welcomedialog
 from . import addons
@@ -160,6 +164,7 @@ class CanvasMainWindow(QMainWindow):
         self.__registry_model = None  # type: Optional[QAbstractItemModel]
         # Proxy widget registry model
         self.__proxy_model = None  # type: Optional[FilterProxyModel]
+        self.__streams = []  # type: List[TextStream]
 
         # TODO: Help view and manager to separate singleton instance.
         self.help = None  # type: HelpManager
@@ -218,7 +223,7 @@ class CanvasMainWindow(QMainWindow):
         w.layout().setContentsMargins(20, 0, 10, 0)
 
         self.scheme_widget = SchemeEditWidget()
-        self.scheme_widget.setScheme(config.workflow_constructor(parent=self))
+        self.set_scheme(config.workflow_constructor(parent=self))
 
         # Save crash recovery swap file on changes to workflow
         self.scheme_widget.undoCommandAdded.connect(self.save_swp)
@@ -267,7 +272,7 @@ class CanvasMainWindow(QMainWindow):
         self.widgets_tool_box.setObjectName("canvas-toolbox")
         self.widgets_tool_box.setTabButtonHeight(30)
         self.widgets_tool_box.setTabIconSize(QSize(26, 26))
-        self.widgets_tool_box.setButtonSize(QSize(64, 84))
+        self.widgets_tool_box.setButtonSize(QSize(68, 84))
         self.widgets_tool_box.setIconSize(QSize(48, 48))
 
         self.widgets_tool_box.triggered.connect(
@@ -944,13 +949,13 @@ class CanvasMainWindow(QMainWindow):
             self.__scheme_margins_enabled = enabled
             self.__update_scheme_margins()
 
-    def scheme_margins_enabled(self):
+    def _scheme_margins_enabled(self):
         # type: () -> bool
         return self.__scheme_margins_enabled
 
-    scheme_margins_enabled = Property(bool,
-                                      fget=scheme_margins_enabled,
-                                      fset=set_scheme_margins_enabled)
+    scheme_margins_enabled: bool
+    scheme_margins_enabled = Property(  # type: ignore
+        bool, _scheme_margins_enabled, set_scheme_margins_enabled)
 
     def __update_scheme_margins(self):
         """Update the margins around the scheme document.
@@ -1028,8 +1033,12 @@ class CanvasMainWindow(QMainWindow):
             stdout.stream.connect(logview.write)
 
         if isinstance(stderr, TextStream):
-            err_formater = logview.formated(color=Qt.red)
-            stderr.stream.connect(err_formater.write)
+            err_formatter = logview.formatted(color=Qt.red)
+            stderr.stream.connect(err_formatter.write)
+
+        for stream in self.__streams:
+            if stream not in (stdout, stderr):
+                window.connect_output_stream(stream)
 
         CanvasMainWindow._instances.append(window)
         window.destroyed.connect(
@@ -1074,8 +1083,6 @@ class CanvasMainWindow(QMainWindow):
             window.freeze_action.setChecked(True)
         window.load_scheme(filename)
 
-        self.ask_load_swp_if_exists()
-
     def open_example_scheme(self, path):  # type: (str) -> None
         # open an workflow without filename/directory tracking.
         if self.is_transient():
@@ -1088,7 +1095,7 @@ class CanvasMainWindow(QMainWindow):
 
         new_scheme = window.new_scheme_from(path)
         if new_scheme is not None:
-            window.set_new_scheme(new_scheme)
+            window.set_scheme(new_scheme)
 
     def _open_workflow_dialog(self):
         # type: () -> QFileDialog
@@ -1169,7 +1176,7 @@ class CanvasMainWindow(QMainWindow):
             mb.open()
 
         if new_scheme is not None:
-            self.set_new_scheme(new_scheme)
+            self.set_scheme(new_scheme, freeze_creation=True)
 
             scheme_doc_widget = self.current_document()
             scheme_doc_widget.setPath(filename)
@@ -1179,6 +1186,12 @@ class CanvasMainWindow(QMainWindow):
                 # activate the default window group.
                 scheme_doc_widget.activateDefaultWindowGroup()
 
+            self.ask_load_swp_if_exists()
+
+            wm = getattr(new_scheme, "widget_manager", None)
+            if wm is not None:
+                wm.set_creation_policy(wm.Normal)
+
     def new_scheme_from(self, filename):
         # type: (str) -> Optional[Scheme]
         """
@@ -1350,19 +1363,19 @@ class CanvasMainWindow(QMainWindow):
             path = recent[0]["path"]
             self.open_scheme_file(path)
 
-    def set_new_scheme(self, new_scheme):
-        # type: (Scheme) -> None
+    def set_scheme(self, new_scheme: Scheme, freeze_creation=False):
         """
         Set new_scheme as the current shown scheme in this window.
 
         The old scheme will be deleted.
         """
-        self.__is_transient = False
-        freeze = self.freeze_action.isChecked()
         scheme_doc = self.current_document()
         old_scheme = scheme_doc.scheme()
+        if old_scheme:
+            self.__is_transient = False
+        freeze_signals = self.freeze_action.isChecked()
         manager = getattr(new_scheme, "signal_manager", None)
-        if freeze and manager is not None:
+        if freeze_signals and manager is not None:
             manager.pause()
         wm = getattr(new_scheme, "widget_manager", None)
         if wm is not None:
@@ -1370,7 +1383,7 @@ class CanvasMainWindow(QMainWindow):
                 self.float_widgets_on_top_action.isChecked()
             )
             wm.set_creation_policy(
-                wm.OnDemand if freeze else wm.Normal
+                wm.OnDemand if freeze_creation else wm.Normal
             )
 
         scheme_doc.setScheme(new_scheme)
@@ -1477,7 +1490,6 @@ class CanvasMainWindow(QMainWindow):
         if filename:
             settings.setValue("last-scheme-dir", os.path.dirname(filename))
             if self.save_scheme_to(curr_scheme, filename):
-                self.clear_swp()
                 document.setPath(filename)
                 document.setModified(False)
                 self.add_recent_scheme(curr_scheme.title, document.path())
@@ -1516,6 +1528,7 @@ class CanvasMainWindow(QMainWindow):
         try:
             with open(filename, "wb") as f:
                 f.write(buffer.getvalue())
+            self.clear_swp()
             return True
         except FileNotFoundError as ex:
             log.error("%s saving '%s'", type(ex).__name__, filename,
@@ -1579,7 +1592,8 @@ class CanvasMainWindow(QMainWindow):
             return
 
         swpname = swp_name(self)
-        self.save_swp_to(swpname)
+        if swpname is not None:
+            self.save_swp_to(swpname)
 
     def save_swp_to(self, filename):
         """
@@ -1594,8 +1608,11 @@ class CanvasMainWindow(QMainWindow):
                     range(undoStack.cleanIndex(), undoStack.count())]
         diff = (propertiesDiff, undoDiff)
 
-        with open(filename, "wb") as f:
-            Pickler(f, document).dump(diff)
+        try:
+            with open(filename, "wb") as f:
+                Pickler(f, document).dump(diff)
+        except Exception:
+            log.error("Could not write swp file %r.", filename, exc_info=True)
 
     def clear_swp(self):
         """
@@ -1618,6 +1635,8 @@ class CanvasMainWindow(QMainWindow):
         Should a swp file for this canvas exist,
         ask the user if they wish to restore changes,
         loading on yes, discarding on no.
+
+        Returns True if swp was loaded, False if not.
         """
         document = self.current_document()
         path = document.path()
@@ -1625,13 +1644,16 @@ class CanvasMainWindow(QMainWindow):
         if path:
             swpname = swp_name(self)
             if not os.path.exists(swpname):
-                return
+                return False
         else:
+            if not QSettings().value('startup/load-crashed-workflows', True, type=bool):
+                return False
             swpnames = glob_scratch_swps()
-            if not swpnames:
-                return
+            if not swpnames or \
+                    all([s in canvas_scratch_name_memo.values() for s in swpnames]):
+                return False
 
-        self.ask_load_swp()
+        return self.ask_load_swp()
 
     def ask_load_swp(self):
         """
@@ -1639,20 +1661,22 @@ class CanvasMainWindow(QMainWindow):
         clearing swp file on no.
         """
         title = self.tr('Restore unsaved changes from crash?')
-
+        name = QApplication.applicationName() or "Orange"
         selected = message_information(
             title,
             self.tr("Restore Changes?"),
-            self.tr("Orange seems to have crashed at some point.\n"
-                    "Changes will be discarded if not restored now."),
+            self.tr("{} seems to have crashed at some point.\n"
+                    "Changes will be discarded if not restored now.").format(name),
             buttons=QMessageBox.Yes | QMessageBox.No,
             default_button=QMessageBox.Yes,
             parent=self)
 
         if selected == QMessageBox.Yes:
             self.load_swp()
+            return True
         elif selected == QMessageBox.No:
             self.clear_swp()
+            return False
         else:
             assert False
 
@@ -1698,11 +1722,25 @@ class CanvasMainWindow(QMainWindow):
         document = self.current_document()
         undoStack = document.undoStack()
 
-        with open(filename, "rb") as f:
-            # type: ({SchemeNode : {}}, [UndoCommand])
-            loaded = Unpickler(f, document.scheme()).load()
+        try:
+            with open(filename, "rb") as f:
+                loaded: Tuple[Dict[SchemeNode, dict], List[UndoCommand]]
+                loaded = Unpickler(f, document.scheme()).load()
+        except Exception:
+            log.error("Could not load swp file: %r", filename, exc_info=True)
+            message_critical(
+                "Could not load restore data.", title="Error", exc_info=True,
+            )
 
-        os.remove(filename)
+            # delete corrupted swp file
+            try:
+                os.remove(filename)
+            except OSError:
+                pass
+
+            return
+
+        register_loaded_swp(self, filename)
 
         document.undoCommandAdded.disconnect(self.save_swp)
 
@@ -2012,15 +2050,17 @@ class CanvasMainWindow(QMainWindow):
         # type: () -> int
         """Open the add-on manager dialog.
         """
+        name = QApplication.applicationName() or "Orange"
         from .addons import have_install_permissions
         if not have_install_permissions():
             QMessageBox(QMessageBox.Warning,
                         "Add-ons: insufficient permissions",
-                        "Insufficient permissions to install add-ons. Try starting Orange "
-                        "as a system administrator or install Orange in user folders.",
+                        "Insufficient permissions to install add-ons. Try starting {name} "
+                        "as a system administrator or install {name} in user folders."
+                        .format(name=name),
                         parent=self).exec_()
         dlg = addons.AddonManagerDialog(
-            self, windowTitle=self.tr("Add-ons"), modal=True
+            self, windowTitle=self.tr("Installer"), modal=True
         )
         dlg.setStyle(QApplication.style())
         dlg.setAttribute(Qt.WA_DeleteOnClose)
@@ -2249,38 +2289,64 @@ class CanvasMainWindow(QMainWindow):
 
         super().showEvent(event)
 
+    def quickHelpEvent(self, event: QuickHelpTipEvent) -> None:
+        if event.priority() == QuickHelpTipEvent.Normal:
+            self.dock_help.showHelp(event.html())
+        elif event.priority() == QuickHelpTipEvent.Temporary:
+            self.dock_help.showHelp(event.html(), event.timeout())
+        elif event.priority() == QuickHelpTipEvent.Permanent:
+            self.dock_help.showPermanentHelp(event.html())
+        event.accept()
+
+    def __handle_help_query_response(self, res: Optional[QUrl]):
+        if res is None:
+            mb = QMessageBox(
+                text=self.tr("There is no documentation for this widget."),
+                windowTitle=self.tr("No help found"),
+                icon=QMessageBox.Information,
+                parent=self,
+                objectName="no-help-found-message-box"
+            )
+            mb.setAttribute(Qt.WA_DeleteOnClose)
+            mb.setWindowModality(Qt.ApplicationModal)
+            mb.show()
+        else:
+            self.show_help(res)
+
+    def whatsThisClickedEvent(self, event: QWhatsThisClickedEvent) -> None:
+        url = QUrl(event.href())
+        if url.scheme() == "help" and url.authority() == "search":
+            loop = get_event_loop()
+            qself = qobjref(self)
+
+            async def run(query_coro: Awaitable[QUrl], query: QUrl):
+                url: Optional[QUrl] = None
+                try:
+                    url = await query_coro
+                except (KeyError, futures.TimeoutError):
+                    log.info("No help topic found for %r", query)
+                self_ = qself()
+                if self_ is not None:
+                    self_.__handle_help_query_response(url)
+            loop.create_task(run(self.help.search_async(url), url))
+        elif url.scheme() == "action" and url.path():
+            action = self.findChild(QAction, url.path())
+            if action is not None:
+                action.trigger()
+            else:
+                log.warning("No target action found for %r", url.toString())
+
     def event(self, event):
         # type: (QEvent) -> bool
         if event.type() == QEvent.StatusTip and \
                 isinstance(event, QuickHelpTipEvent):
-            if event.priority() == QuickHelpTipEvent.Normal:
-                self.dock_help.showHelp(event.html())
-            elif event.priority() == QuickHelpTipEvent.Temporary:
-                self.dock_help.showHelp(event.html(), event.timeout())
-            elif event.priority() == QuickHelpTipEvent.Permanent:
-                self.dock_help.showPermanentHelp(event.html())
-            return True
-
+            self.quickHelpEvent(event)
+            if event.isAccepted():
+                return True
         elif event.type() == QEvent.WhatsThisClicked:
             event = cast(QWhatsThisClickedEvent, event)
-            url = QUrl(event.href())
-            if url.scheme() == "help" and url.authority() == "search":
-                try:
-                    url = self.help.search(url)
-                    self.show_help(url)
-                except KeyError:
-                    log.info("No help topic found for %r", url)
-                    message_information(
-                        self.tr("There is no documentation for this widget."),
-                        parent=self)
-            elif url.scheme() == "action" and url.path():
-                action = self.findChild(QAction, url.path())
-                if action is not None:
-                    action.trigger()
-                else:
-                    log.warning("No target action found for %r", url.toString())
+            self.whatsThisClickedEvent(event)
             return True
-
         return super().event(event)
 
     def show_help(self, url):
@@ -2422,13 +2488,30 @@ class CanvasMainWindow(QMainWindow):
                     category_filter_function(visible_state))
             ])
 
+    def connect_output_stream(self, stream: TextStream):
+        """
+        Connect a :class:`TextStream` instance to this window's output view.
+
+        The `stream` will be 'inherited' by new windows created by
+        `create_new_window`.
+        """
+        self.__streams.append(stream)
+        view = self.output_view()
+        stream.stream.connect(view.write)
+
+    def disconnect_output_stream(self, stream: TextStream):
+        """
+        Disconnect a :class:`TextStream` instance from this window's
+        output view.
+        """
+        self.__streams.remove(stream)
+        view = self.output_view()
+        stream.stream.disconnect(view.write)
+
 
 def updated_flags(flags, mask, state):
-    if state:
-        flags |= mask
-    else:
-        flags &= ~mask
-    return flags
+    return set_flag(flags, mask, state)
+
 
 
 def identity(item):
