@@ -4,6 +4,7 @@ Node Item
 =========
 
 """
+import math
 import typing
 import string
 
@@ -11,36 +12,40 @@ from operator import attrgetter
 from itertools import groupby
 from xml.sax.saxutils import escape
 
-from typing import Dict, Any, Optional, List, Iterable, Tuple
+from typing import Dict, Any, Optional, List, Iterable, Tuple, Union
 
 from AnyQt.QtWidgets import (
-    QGraphicsItem, QGraphicsObject, QGraphicsTextItem, QGraphicsWidget,
+    QGraphicsItem, QGraphicsObject, QGraphicsWidget,
     QGraphicsDropShadowEffect, QStyle, QApplication, QGraphicsSceneMouseEvent,
     QGraphicsSceneContextMenuEvent, QStyleOptionGraphicsItem, QWidget,
     QGraphicsEllipseItem
 )
 from AnyQt.QtGui import (
     QPen, QBrush, QColor, QPalette, QIcon, QPainter, QPainterPath,
-    QPainterPathStroker, QTextDocument, QTextBlock, QTextLine,
-    QConicalGradient
-)
+    QPainterPathStroker, QConicalGradient,
+    QTransform)
 from AnyQt.QtCore import (
     Qt, QEvent, QPointF, QRectF, QRect, QSize, QTime, QTimer,
-    QPropertyAnimation, QEasingCurve, QObject, QVariantAnimation
-)
+    QPropertyAnimation, QEasingCurve, QObject, QVariantAnimation,
+    QParallelAnimationGroup)
 from AnyQt.QtCore import pyqtSignal as Signal, pyqtProperty as Property
+from PyQt5.QtCore import pyqtProperty
 
 from .graphicspathobject import GraphicsPathObject
+from .graphicstextitem import GraphicsTextItem
 from .utils import saturated, radial_gradient
 
 from ...scheme.node import UserMessage
-from ...registry import NAMED_COLORS, WidgetDescription, CategoryDescription
+from ...registry import NAMED_COLORS, WidgetDescription, CategoryDescription, \
+    InputSignal, OutputSignal
 from ...resources import icon_loader
 from .utils import uniform_linear_layout_trunc
+from ...utils import set_flag
+from ...utils.mathutils import interp1d
 
 if typing.TYPE_CHECKING:
     from ...registry import WidgetDescription
-    from . import LinkItem
+    # from . import LinkItem
 
 
 def create_palette(light_color, color):
@@ -287,12 +292,9 @@ class NodeBodyItem(GraphicsPathObject):
 
         self.shadow.setColor(color)
 
-        if radius == self.shadow.blurRadius():
-            return
-
         if self.__animationEnabled:
             if self.__blurAnimation.state() == QPropertyAnimation.Running:
-                self.__blurAnimation.pause()
+                self.__blurAnimation.stop()
 
             self.__blurAnimation.setStartValue(self.shadow.blurRadius())
             self.__blurAnimation.setEndValue(radius)
@@ -344,6 +346,8 @@ class LinkAnchorIndicator(QGraphicsEllipseItem):
     """
     def __init__(self, parent=None):
         # type: (Optional[QGraphicsItem]) -> None
+        self.__styleState = QStyle.State(0)
+        self.__linkState = LinkItem.NoState
         super().__init__(parent)
         self.setAcceptedMouseButtons(Qt.NoButton)
         self.setRect(-3.5, -3.5, 7., 7.)
@@ -358,13 +362,25 @@ class LinkAnchorIndicator(QGraphicsEllipseItem):
         """
         The hover state is set by the LinkItem.
         """
-        if self.__hover != state:
-            self.__hover = state
+        state = set_flag(self.__styleState, QStyle.State_MouseOver, state)
+        self.setStyleState(state)
+
+    def setStyleState(self, state: QStyle.State):
+        if self.__styleState != state:
+            self.__styleState = state
+            self.update()
+
+    def setLinkState(self, state: 'LinkItem.State'):
+        if self.__linkState != state:
+            self.__linkState = state
             self.update()
 
     def paint(self, painter, option, widget=None):
         # type: (QPainter, QStyleOptionGraphicsItem, Optional[QWidget]) -> None
-        brush = self.hoverBrush if self.__hover else self.brush()
+        hover = self.__styleState & (QStyle.State_Selected | QStyle.State_MouseOver)
+        brush = self.hoverBrush if hover else self.brush()
+        if self.__linkState & (LinkItem.Pending | LinkItem.Invalidated):
+            brush = QBrush(Qt.red)
 
         painter.setBrush(brush)
         painter.setPen(self.pen())
@@ -410,16 +426,32 @@ class AnchorPoint(QGraphicsObject):
     #: Signal emitted when the item's `anchorDirection` changes.
     anchorDirectionChanged = Signal(QPointF)
 
-    def __init__(self, parent=None, **kwargs):
-        # type: (Optional[QGraphicsItem], Any) -> None
+    #: Signal emitted when anchor's Input/Output channel changes.
+    signalChanged = Signal(QGraphicsObject)
+
+    def __init__(
+            self,
+            parent: Optional[QGraphicsItem] = None,
+            signal: Union[InputSignal, OutputSignal, None] = None,
+            **kwargs
+    ) -> None:
         super().__init__(parent, **kwargs)
         self.setFlag(QGraphicsItem.ItemIsFocusable)
         self.setFlag(QGraphicsItem.ItemSendsScenePositionChanges, True)
         self.setFlag(QGraphicsItem.ItemHasNoContents, True)
         self.indicator = LinkAnchorIndicator(self)
 
+        self.signal = signal
         self.__direction = QPointF()
 
+        self.anim = QPropertyAnimation(self, b'pos', self)
+        self.anim.setDuration(50)
+
+    def setSignal(self, signal):
+        if self.signal != signal:
+            self.signal = signal
+            self.signalChanged.emit(self)
+
     def anchorScenePos(self):
         # type: () -> QPointF
         """
@@ -456,8 +488,109 @@ class AnchorPoint(QGraphicsObject):
     def setHoverState(self, enabled):
         self.indicator.setHoverState(enabled)
 
-    def setBrush(self, brush):
-        self.indicator.setBrush(brush)
+    def setLinkState(self, state: 'LinkItem.State'):
+        self.indicator.setLinkState(state)
+
+
+def drawDashPattern(dashNum, spaceLen=2, lineLen=16):
+    dashLen = (lineLen - spaceLen * (dashNum - 1)) / dashNum
+    line = []
+    for _ in range(dashNum - 1):
+        line += [dashLen, spaceLen]
+    line += [dashLen]
+    return line
+
+
+def matchDashPattern(l1, l2, spaceLen=2):
+    if not l1 or not l2 or len(l1) == len(l2):
+        return l1, l2
+
+    if len(l2) < len(l1):
+        l1, l2 = l2, l1
+        reverse = True
+    else:
+        reverse = False
+
+    l1d = len(l1) // 2 + 1
+    l2d = len(l2) // 2 + 1
+
+    if l1d == 1:  # base case
+        dLen = l1[0]
+        l1 = drawDashPattern(l2d, spaceLen=0, lineLen=dLen)
+        return (l2, l1) if reverse else (l1, l2)
+
+    d = math.gcd(l1d, l2d)
+    if d > 1:  # split
+        l1step = (l1d // d) * 2
+        l2step = (l2d // d) * 2
+        l1range = l1step - 1
+        l2range = l2step - 1
+        l1splits, l2splits = [], []
+        for l1i, l2i in zip(range(0, len(l1), l1step), range(0, len(l2), l2step)):
+            l1s = l1[l1i:(l1i+l1range)]
+            l2s = l2[l2i:(l2i+l2range)]
+            l1splits += [l1s]
+            l2splits += [l2s]
+
+    elif l1d % 2 == 0 and l2d % 2 != 0:  # split middle 2 lines into 3
+        l11 = l1[:l1d-2]
+        l1l = l1[l1d]
+        l12 = l1[l1d+1:]
+
+        l21 = l2[:l2d-3]
+        l2l = l2[l2d-1]
+        l22 = l2[l2d+2:]
+
+        new_l11, new_l21 = matchDashPattern(l11, l21)
+        new_l12, new_l22 = matchDashPattern(l12, l22)
+        for new_l in (new_l11, new_l21):
+            if new_l:
+                new_l += [spaceLen]
+        for new_l in (new_l12, new_l22):
+            if new_l:
+                new_l.insert(0, spaceLen)
+
+        l1 = new_l11 + [l1l*2/3, 0, l1l/3, spaceLen, l1l/3, 0, l1l*2/3] + new_l12
+        l2 = new_l21 + [l2l, spaceLen, l2l/2, 0, l2l/2, spaceLen, l2l] + new_l22
+        return (l2, l1) if reverse else (l1, l2)
+
+    elif l1d % 2 != 0 and l2d % 2 == 0:  # split line
+        l11 = l1[:l1d - 2]
+        mid = l1[l1d-1]
+        l1m = [mid/2, 0, mid/2]
+        l12 = l1[l1d+1:]
+
+        l21 = l2[:l2d-3]
+        l2m = l2[l2d-2:l2d+1]
+        l22 = l2[l2d+2:]
+
+        l1splits = [l11, l1m, l12]
+        l2splits = [l21, l2m, l22]
+    else:  # if l1d % 2 != 0 and l2d % 2 != 0
+        l11 = l1[:l1d - 1]
+        l1m = l1[l1d]
+        l12 = l1[l1d + 2:]
+
+        l21 = l2[:l2d - 1]
+        l2m = l2[l2d]
+        l22 = l2[l2d + 2:]
+
+        l1splits = [l11, l1m, l12]
+        l2splits = [l21, l2m, l22]
+
+    l1 = []
+    l2 = []
+    for l1s, l2s in zip(l1splits, l2splits):
+        new_l1, new_l2 = matchDashPattern(l1s, l2s)
+        l1 += new_l1 + [spaceLen]
+        l2 += new_l2 + [spaceLen]
+    # drop trailing space
+    l1 = l1[:-1]
+    l2 = l2[:-1]
+    return (l2, l1) if reverse else (l1, l2)
+
+
+ANCHOR_TEXT_MARGIN = 4
 
 
 class NodeAnchorItem(GraphicsPathObject):
@@ -478,6 +611,9 @@ class NodeAnchorItem(GraphicsPathObject):
 
         self.__animationEnabled = False
         self.__hover = False
+        self.__anchorOpen = False
+        self.__compatibleSignals = None
+        self.__keepSignalsOpen = []
 
         # Does this item have any anchored links.
         self.anchored = False
@@ -489,10 +625,16 @@ class NodeAnchorItem(GraphicsPathObject):
 
         self.__anchorPath = QPainterPath()
         self.__points = []  # type: List[AnchorPoint]
-        self.__pointPositions = []  # type: List[float]
+        self.__uniformPointPositions = []  # type: List[float]
+        self.__channelPointPositions = []  # type: List[float]
+        self.__incompatible = False  # type: bool
+        self.__signals = []  # type: List[Union[InputSignal, OutputSignal]]
+        self.__signalLabels = []  # type: List[GraphicsTextItem]
+        self.__signalLabelAnims = []  # type: List[QPropertyAnimation]
 
         self.__fullStroke = QPainterPath()
         self.__dottedStroke = QPainterPath()
+        self.__channelStroke = QPainterPath()
         self.__shape = None  # type: Optional[QPainterPath]
 
         self.shadow = QGraphicsDropShadowEffect(
@@ -514,6 +656,72 @@ class NodeAnchorItem(GraphicsPathObject):
         self.__blurAnimation.setDuration(50)
         self.__blurAnimation.finished.connect(self.__on_finished)
 
+        stroke_path = QPainterPathStroker()
+        stroke_path.setCapStyle(Qt.RoundCap)
+        stroke_path.setWidth(3)
+        self.__pathStroker = stroke_path
+        self.__interpDash = None
+        self.__dashInterpFactor = 0
+        self.__anchorPathAnim = QPropertyAnimation(self,
+                                                   b"anchorDashInterpFactor",
+                                                   self)
+        self.__anchorPathAnim.setDuration(50)
+
+        self.animGroup = QParallelAnimationGroup()
+        self.animGroup.addAnimation(self.__anchorPathAnim)
+
+    def setSignals(self, signals):
+        self.__signals = signals
+        self.setAnchorPath(self.__anchorPath)  # (re)instantiate anchor paths
+
+        # TODO this is ugly
+        alignLeft = isinstance(self, SourceAnchorItem)
+
+        for s in signals:
+            lbl = GraphicsTextItem(self)
+            lbl.setAcceptedMouseButtons(Qt.NoButton)
+            lbl.setAcceptHoverEvents(False)
+
+            text = s.name
+            lbl.setHtml('<div align="' + ('left' if alignLeft else 'right') +
+                        '" style="font-size: small; background-color: palette(base);" >{0}</div>'
+                        .format(text))
+
+            cperc = self.__getChannelPercent(s)
+            sigPos = self.__anchorPath.pointAtPercent(cperc)
+            lblrect = lbl.boundingRect()
+
+            transform = QTransform()
+            transform.translate(sigPos.x(), sigPos.y())
+            transform.translate(0, -lblrect.height() / 2)
+            if not alignLeft:
+                transform.translate(-lblrect.width() - ANCHOR_TEXT_MARGIN, 0)
+            else:
+                transform.translate(ANCHOR_TEXT_MARGIN, 0)
+
+            lbl.setTransform(transform)
+            lbl.setOpacity(0)
+            self.__signalLabels.append(lbl)
+
+            lblAnim = QPropertyAnimation(lbl, b'opacity', self)
+            lblAnim.setDuration(50)
+            self.animGroup.addAnimation(lblAnim)
+            self.__signalLabelAnims.append(lblAnim)
+
+    def setIncompatible(self, enabled):
+        if self.__incompatible != enabled:
+            self.__incompatible = enabled
+            self.__updatePositions()
+
+    def setKeepAnchorOpen(self, signal):
+        if signal is None:
+            self.__keepSignalsOpen = []
+        elif not isinstance(signal, list):
+            self.__keepSignalsOpen = [signal]
+        else:
+            self.__keepSignalsOpen = signal
+        self.__updateLabels(self.__keepSignalsOpen)
+
     def parentNodeItem(self):
         # type: () -> Optional['NodeItem']
         """
@@ -536,24 +744,39 @@ class NodeAnchorItem(GraphicsPathObject):
         stroke_path.setWidth(25)
         self.prepareGeometryChange()
         self.__shape = stroke_path.createStroke(path)
+        stroke_path.setWidth(3)
+
+        # Match up dash patterns for animations
+        dash6 = drawDashPattern(6)
+        channelAnchor = drawDashPattern(len(self.__signals) or 1)
+        fullAnchor = drawDashPattern(1)
+        dash6, channelAnchor = matchDashPattern(dash6, channelAnchor)
+        channelAnchor, fullAnchor = matchDashPattern(channelAnchor, fullAnchor)
+        self.__unanchoredDash = dash6
+        self.__channelDash = channelAnchor
+        self.__anchoredDash = fullAnchor
 
         # The full stroke
-        stroke_path.setWidth(3)
+        stroke_path.setDashPattern(fullAnchor)
         self.__fullStroke = stroke_path.createStroke(path)
 
         # The dotted stroke (when not connected to anything)
-        stroke_path.setDashPattern(Qt.DotLine)
+        stroke_path.setDashPattern(dash6)
         self.__dottedStroke = stroke_path.createStroke(path)
 
+        # The channel stroke (when channels are open)
+        stroke_path.setDashPattern(channelAnchor)
+        self.__channelStroke = stroke_path.createStroke(path)
+
         if self.anchored:
-            assert self.__fullStroke is not None
             self.setPath(self.__fullStroke)
+            self.__pathStroker.setDashPattern(self.__anchoredDash)
             self.__shadow.setPath(self.__fullStroke)
             brush = self.connectedHoverBrush if self.__hover else self.connectedBrush
             self.setBrush(brush)
         else:
-            assert self.__dottedStroke is not None
             self.setPath(self.__dottedStroke)
+            self.__pathStroker.setDashPattern(self.__unanchoredDash)
             self.__shadow.setPath(self.__dottedStroke)
             brush = self.normalHoverBrush if self.__hover else self.normalBrush
             self.setBrush(brush)
@@ -566,6 +789,25 @@ class NodeAnchorItem(GraphicsPathObject):
         """
         return QPainterPath(self.__anchorPath)
 
+    def anchorOpen(self):
+        return self.__anchorOpen
+
+    @pyqtProperty(float)
+    def anchorDashInterpFactor(self):
+        return self.__dashInterpFactor
+
+    @anchorDashInterpFactor.setter
+    def anchorDashInterpFactor(self, value):
+        self.__dashInterpFactor = value
+        stroke_path = self.__pathStroker
+        path = self.__anchorPath
+
+        pattern = self.__interpDash(value)
+        stroke_path.setDashPattern(pattern)
+        stroke = stroke_path.createStroke(path)
+        self.setPath(stroke)
+        self.__shadow.setPath(stroke)
+
     def setAnchored(self, anchored):
         # type: (bool) -> None
         """
@@ -574,16 +816,12 @@ class NodeAnchorItem(GraphicsPathObject):
         """
         self.anchored = anchored
         if anchored:
-            self.setPath(self.__fullStroke)
-            self.__shadow.setPath(self.__fullStroke)
-            hover = self.__hover and len(self.__points) > 1  # a stylistic choice
-            brush = self.connectedHoverBrush if hover else self.connectedBrush
-            self.setBrush(brush)
+            self.shadow.setEnabled(False)
+            self.setBrush(self.connectedBrush)
         else:
-            self.setPath(self.__dottedStroke)
-            self.__shadow.setPath(self.__dottedStroke)
             brush = self.normalHoverBrush if self.__hover else self.normalBrush
             self.setBrush(brush)
+        self.__updatePositions()
 
     def setConnectionHint(self, hint=None):
         """
@@ -600,8 +838,8 @@ class NodeAnchorItem(GraphicsPathObject):
         """
         return len(self.__points)
 
-    def addAnchor(self, anchor, position=0.5):
-        # type: (AnchorPoint, float) -> int
+    def addAnchor(self, anchor):
+        # type: (AnchorPoint) -> int
         """
         Add a new :class:`AnchorPoint` to this item and return it's index.
 
@@ -609,10 +847,16 @@ class NodeAnchorItem(GraphicsPathObject):
         point inserted.
 
         """
-        return self.insertAnchor(self.count(), anchor, position)
+        return self.insertAnchor(self.count(), anchor)
 
-    def insertAnchor(self, index, anchor, position=0.5):
-        # type: (int, AnchorPoint, float) -> int
+    def __updateAnchorSignalPosition(self, anchor):
+        cperc = self.__getChannelPercent(anchor.signal)
+        i = self.__points.index(anchor)
+        self.__channelPointPositions[i] = cperc
+        self.__updatePositions()
+
+    def insertAnchor(self, index, anchor):
+        # type: (int, AnchorPoint) -> int
         """
         Insert a new :class:`AnchorPoint` at `index`.
 
@@ -625,24 +869,32 @@ class NodeAnchorItem(GraphicsPathObject):
             raise ValueError("%s already added." % anchor)
 
         self.__points.insert(index, anchor)
-        self.__pointPositions.insert(index, position)
+        self.__uniformPointPositions.insert(index, 0)
+        cperc = self.__getChannelPercent(anchor.signal)
+        self.__channelPointPositions.insert(index, cperc)
+        self.animGroup.addAnimation(anchor.anim)
 
         anchor.setParentItem(self)
-        anchor.setPos(self.__anchorPath.pointAtPercent(position))
         anchor.destroyed.connect(self.__onAnchorDestroyed)
+        anchor.signalChanged.connect(self.__updateAnchorSignalPosition)
 
-        self.__updatePositions()
+        positions = self.anchorPositions()
+        positions = uniform_linear_layout_trunc(positions)
+
+        if anchor.signal in self.__keepSignalsOpen or \
+                self.__anchorOpen and self.__hover:
+            perc = cperc
+        else:
+            perc = positions[index]
+        pos = self.__anchorPath.pointAtPercent(perc)
+        anchor.setPos(pos)
+
+        self.setAnchorPositions(positions)
 
         self.setAnchored(bool(self.__points))
 
         hover_for_color = self.__hover and len(self.__points) > 1  # a stylistic choice
         anchor.setHoverState(hover_for_color)
-
-        # when an anchor is added, the anchor is no longer hovered
-        # the color should stay the same, but the shadow should fade out
-        self.__hover = False
-        self.__updateShadowState()
-
         return index
 
     def removeAnchor(self, anchor):
@@ -651,11 +903,16 @@ class NodeAnchorItem(GraphicsPathObject):
         Remove and delete the anchor point.
         """
         anchor = self.takeAnchor(anchor)
+        self.animGroup.removeAnimation(anchor.anim)
 
         anchor.hide()
         anchor.setParentItem(None)
         anchor.deleteLater()
 
+        positions = self.anchorPositions()
+        positions = uniform_linear_layout_trunc(positions)
+        self.setAnchorPositions(positions)
+
     def takeAnchor(self, anchor):
         # type: (AnchorPoint) -> AnchorPoint
         """
@@ -664,7 +921,8 @@ class NodeAnchorItem(GraphicsPathObject):
         index = self.__points.index(anchor)
 
         del self.__points[index]
-        del self.__pointPositions[index]
+        del self.__uniformPointPositions[index]
+        del self.__channelPointPositions[index]
 
         anchor.destroyed.disconnect(self.__onAnchorDestroyed)
 
@@ -682,7 +940,8 @@ class NodeAnchorItem(GraphicsPathObject):
             return
 
         del self.__points[index]
-        del self.__pointPositions[index]
+        del self.__uniformPointPositions[index]
+        del self.__channelPointPositions[index]
 
     def anchorPoints(self):
         # type: () -> List[AnchorPoint]
@@ -703,9 +962,8 @@ class NodeAnchorItem(GraphicsPathObject):
         """
         Set the anchor positions in percentages (0..1) along the path curve.
         """
-        if self.__pointPositions != positions:
-            self.__pointPositions = list(positions)
-
+        if self.__uniformPointPositions != positions:
+            self.__uniformPointPositions = list(positions)
             self.__updatePositions()
 
     def anchorPositions(self):
@@ -715,7 +973,7 @@ class NodeAnchorItem(GraphicsPathObject):
         each float is between 0 and 1 and specifies where along the anchor
         path does the point lie (0 is at start 1 is at the end).
         """
-        return list(self.__pointPositions)
+        return list(self.__uniformPointPositions)
 
     def shape(self):
         # type: () -> QPainterPath
@@ -730,18 +988,21 @@ class NodeAnchorItem(GraphicsPathObject):
         else:
             return GraphicsPathObject.boundingRect(self)
 
-    def hoverEnterEvent(self, event):
-        self.__hover = True
-        brush = self.connectedHoverBrush if self.anchored else self.normalHoverBrush
+    def setHovered(self, enabled):
+        self.__hover = enabled
+        if enabled:
+            brush = self.connectedHoverBrush if self.anchored else self.normalHoverBrush
+        else:
+            brush = self.connectedBrush if self.anchored else self.normalBrush
         self.setBrush(brush)
         self.__updateHoverState()
+
+    def hoverEnterEvent(self, event):
+        self.setHovered(True)
         return super().hoverEnterEvent(event)
 
     def hoverLeaveEvent(self, event):
-        self.__hover = False
-        brush = self.connectedBrush if self.anchored else self.normalBrush
-        self.setBrush(brush)
-        self.__updateHoverState()
+        self.setHovered(False)
         return super().hoverLeaveEvent(event)
 
     def setAnimationEnabled(self, enabled):
@@ -752,12 +1013,35 @@ class NodeAnchorItem(GraphicsPathObject):
         if self.__animationEnabled != enabled:
             self.__animationEnabled = enabled
 
+    def signalAtPos(self, scenePos, signalsToFind=None):
+        if signalsToFind is None:
+            signalsToFind = self.__signals
+        pos = self.mapFromScene(scenePos)
+
+        def signalLengthToPos(s):
+            perc = self.__getChannelPercent(s)
+            p = self.__anchorPath.pointAtPercent(perc)
+            return (p - pos).manhattanLength()
+
+        return min(signalsToFind, key=signalLengthToPos)
+
     def __updateHoverState(self):
         self.__updateShadowState()
+        self.__updatePositions()
 
         for indicator in self.anchorPoints():
             indicator.setHoverState(self.__hover)
 
+    def __getChannelPercent(self, signal):
+        if signal is None:
+            return 0.5
+        signals = self.__signals
+
+        ci = signals.index(signal)
+        gap_perc = 1 / 8
+        seg_perc = (1 - (gap_perc * (len(signals) - 1))) / len(signals)
+        return (ci * (gap_perc + seg_perc)) + seg_perc / 2
+
     def __updateShadowState(self):
         # type: () -> None
         radius = 5 if self.__hover else 0
@@ -767,7 +1051,7 @@ class NodeAnchorItem(GraphicsPathObject):
 
         if self.__animationEnabled:
             if self.__blurAnimation.state() == QPropertyAnimation.Running:
-                self.__blurAnimation.pause()
+                self.__blurAnimation.stop()
 
             self.__blurAnimation.setStartValue(self.shadow.blurRadius())
             self.__blurAnimation.setEndValue(radius)
@@ -775,13 +1059,76 @@ class NodeAnchorItem(GraphicsPathObject):
         else:
             self.shadow.setBlurRadius(radius)
 
+    def setAnchorOpen(self, anchorOpen):
+        self.__anchorOpen = anchorOpen
+        self.__updatePositions()
+
+    def setCompatibleSignals(self, compatibleSignals):
+        self.__compatibleSignals = compatibleSignals
+        self.__updatePositions()
+
+    def __updateLabels(self, showSignals):
+        for signal, label in zip(self.__signals, self.__signalLabels):
+            if signal not in showSignals:
+                opacity = 0
+            elif self.__compatibleSignals is not None \
+                    and signal not in self.__compatibleSignals:
+                opacity = 0.65
+            else:
+                opacity = 1
+            label.setOpacity(opacity)
+
+    def __initializeAnimation(self, targetPoss, endDash, showSignals):
+        anchorOpen = self.__anchorOpen
+        # TODO if animation currently running, set start value/time accordingly
+        for a, t in zip(self.__points, targetPoss):
+            currPos = a.pos()
+            a.anim.setStartValue(currPos)
+            pos = self.__anchorPath.pointAtPercent(t)
+            a.anim.setEndValue(pos)
+
+        for sig, lbl, lblAnim in zip(self.__signals, self.__signalLabels, self.__signalLabelAnims):
+            lblAnim.setStartValue(lbl.opacity())
+            lblAnim.setEndValue(1 if sig in showSignals else 0)
+
+        startDash = self.__pathStroker.dashPattern()
+        self.__interpDash = interp1d(startDash, endDash)
+        self.__anchorPathAnim.setStartValue(0)
+        self.__anchorPathAnim.setEndValue(1)
+
     def __updatePositions(self):
         # type: () -> None
         """Update anchor points positions.
         """
-        for point, t in zip(self.__points, self.__pointPositions):
-            pos = self.__anchorPath.pointAtPercent(t)
-            point.setPos(pos)
+        if self.__keepSignalsOpen or self.__anchorOpen and self.__hover:
+            dashPattern = self.__channelDash
+            stroke = self.__channelStroke
+            targetPoss = self.__channelPointPositions
+            showSignals = self.__keepSignalsOpen or self.__signals
+        elif self.anchored:
+            dashPattern = self.__anchoredDash
+            stroke = self.__fullStroke
+            targetPoss = self.__uniformPointPositions
+            showSignals = self.__signals if self.__incompatible else []
+        else:
+            dashPattern = self.__unanchoredDash
+            stroke = self.__dottedStroke
+            targetPoss = self.__uniformPointPositions
+            showSignals = self.__signals if self.__incompatible else []
+
+        if self.animGroup.state() == QPropertyAnimation.Running:
+            self.animGroup.stop()
+        if self.__animationEnabled:
+            self.__initializeAnimation(targetPoss, dashPattern, showSignals)
+            self.animGroup.start()
+        else:
+            for point, t in zip(self.__points, targetPoss):
+                pos = self.__anchorPath.pointAtPercent(t)
+                point.setPos(pos)
+            self.__updateLabels(showSignals)
+            self.__pathStroker.setDashPattern(dashPattern)
+            self.setPath(stroke)
+            self.__shadow.setPath(stroke)
 
     def __on_finished(self):
         # type: () -> None
@@ -919,105 +1266,6 @@ class GraphicsIconItem(QGraphicsWidget):
             self.__icon.paint(painter, target, Qt.AlignCenter, mode)
 
 
-class NameTextItem(QGraphicsTextItem):
-    def __init__(self, *args, **kwargs):
-        # type: (Any, Any) -> None
-        super().__init__(*args, **kwargs)
-        self.__selected = False
-        self.__palette = None  # type: Optional[QPalette]
-        self.__content = ""
-        #: The cached text background shape when this item is selected
-        self.__cachedBackgroundPath = None  # type: Optional[QPainterPath]
-        self.document().documentLayoutChanged.connect(self.__onLayoutChanged)
-
-    def __onLayoutChanged(self):
-        self.__cachedBackgroundPath = None
-        self.update()
-
-    def paint(self, painter, option, widget=None):
-        # type: (QPainter, QStyleOptionGraphicsItem, Optional[QWidget]) -> None
-        if self.__selected:
-            path = self.__textBackgroundPath()
-            painter.save()
-            painter.setPen(QPen(Qt.NoPen))
-            painter.setBrush(self.palette().color(QPalette.Highlight))
-            painter.drawPath(path)
-            painter.restore()
-
-        super().paint(painter, option, widget)
-
-    def _blocks(self, doc):
-        # type: (QTextDocument) -> Iterable[QTextBlock]
-        block = doc.begin()
-        while block != doc.end():
-            yield block
-            block = block.next()
-
-    def _lines(self, doc):
-        # type: (QTextDocument) -> Iterable[QTextLine]
-        for block in self._blocks(doc):
-            blocklayout = block.layout()
-            for i in range(blocklayout.lineCount()):
-                yield blocklayout.lineAt(i)
-
-    def __textBackgroundPath(self) -> QPainterPath:
-        # return a path outlining all the text lines.
-        if self.__cachedBackgroundPath is None:
-            doc = self.document()
-            margin = doc.documentMargin()
-            path = QPainterPath()
-            offset = min(margin, 2)
-            for line in self._lines(doc):
-                rect = line.naturalTextRect()
-                rect.translate(margin, margin)
-                rect = rect.adjusted(-offset, -offset, offset, offset)
-                p = QPainterPath()
-                p.addRoundedRect(rect, 3, 3)
-                path = path.united(p)
-            self.__cachedBackgroundPath = path
-        return self.__cachedBackgroundPath
-
-    def setSelectionState(self, state):
-        # type: (bool) -> None
-        if self.__selected != state:
-            self.__selected = state
-            self.__updateDefaultTextColor()
-            self.update()
-
-    def setPalette(self, palette):
-        # type: (QPalette) -> None
-        if self.__palette != palette:
-            self.__palette = QPalette(palette)
-            self.__updateDefaultTextColor()
-            self.update()
-
-    def palette(self):
-        # type: () -> QPalette
-        if self.__palette is None:
-            scene = self.scene()
-            if scene is not None:
-                return scene.palette()
-            else:
-                return QPalette()
-        else:
-            return QPalette(self.__palette)
-
-    def __updateDefaultTextColor(self):
-        # type: () -> None
-        if self.__selected:
-            role = QPalette.HighlightedText
-        else:
-            role = QPalette.WindowText
-        self.setDefaultTextColor(self.palette().color(role))
-
-    def setHtml(self, contents):
-        # type: (str) -> None
-        if contents != self.__content:
-            self.__content = contents
-            self.__cachedBackgroundPath = None
-            super().setHtml(contents)
-
-
 class NodeItem(QGraphicsWidget):
     """
     An widget node item in the canvas.
@@ -1036,6 +1284,9 @@ class NodeItem(QGraphicsWidget):
     #: The item is under the mouse.
     hovered = Signal()
 
+    #: Signal emitted the the item's selection state changes.
+    selectedChanged = Signal(bool)
+
     #: Span of the anchor in degrees
     ANCHOR_SPAN_ANGLE = 90
 
@@ -1098,8 +1349,7 @@ class NodeItem(QGraphicsWidget):
         self.outputAnchorItem.hide()
 
         # Title caption item
-        self.captionTextItem = NameTextItem(self)
-
+        self.captionTextItem = GraphicsTextItem(self)
         self.captionTextItem.setPlainText("")
         self.captionTextItem.setPos(0, 33)
 
@@ -1163,8 +1413,10 @@ class NodeItem(QGraphicsWidget):
             self.setTitle(desc.name)
 
         if desc.inputs:
+            self.inputAnchorItem.setSignals(desc.inputs)
             self.inputAnchorItem.show()
         if desc.outputs:
+            self.outputAnchorItem.setSignals(desc.outputs)
             self.outputAnchorItem.show()
 
         tooltip = NodeItem_toolTipHelper(self)
@@ -1334,20 +1586,16 @@ class NodeItem(QGraphicsWidget):
             self.__info = message
             self.__updateMessages()
 
-    def newInputAnchor(self):
-        # type: () -> AnchorPoint
+    def newInputAnchor(self, signal=None):
+        # type: (Optional[InputSignal]) -> AnchorPoint
         """
         Create and return a new input :class:`AnchorPoint`.
         """
         if not (self.widget_description and self.widget_description.inputs):
             raise ValueError("Widget has no inputs.")
 
-        anchor = AnchorPoint()
-        self.inputAnchorItem.addAnchor(anchor, position=1.0)
-
-        positions = self.inputAnchorItem.anchorPositions()
-        positions = uniform_linear_layout_trunc(positions)
-        self.inputAnchorItem.setAnchorPositions(positions)
+        anchor = AnchorPoint(self, signal=signal)
+        self.inputAnchorItem.addAnchor(anchor)
 
         return anchor
 
@@ -1358,24 +1606,16 @@ class NodeItem(QGraphicsWidget):
         """
         self.inputAnchorItem.removeAnchor(anchor)
 
-        positions = self.inputAnchorItem.anchorPositions()
-        positions = uniform_linear_layout_trunc(positions)
-        self.inputAnchorItem.setAnchorPositions(positions)
-
-    def newOutputAnchor(self):
-        # type: () -> AnchorPoint
+    def newOutputAnchor(self, signal=None):
+        # type: (Optional[OutputSignal]) -> AnchorPoint
         """
         Create and return a new output :class:`AnchorPoint`.
         """
         if not (self.widget_description and self.widget_description.outputs):
             raise ValueError("Widget has no outputs.")
 
-        anchor = AnchorPoint(self)
-        self.outputAnchorItem.addAnchor(anchor, position=1.0)
-
-        positions = self.outputAnchorItem.anchorPositions()
-        positions = uniform_linear_layout_trunc(positions)
-        self.outputAnchorItem.setAnchorPositions(positions)
+        anchor = AnchorPoint(self, signal=signal)
+        self.outputAnchorItem.addAnchor(anchor)
 
         return anchor
 
@@ -1386,10 +1626,6 @@ class NodeItem(QGraphicsWidget):
         """
         self.outputAnchorItem.removeAnchor(anchor)
 
-        positions = self.outputAnchorItem.anchorPositions()
-        positions = uniform_linear_layout_trunc(positions)
-        self.outputAnchorItem.setAnchorPositions(positions)
-
     def inputAnchors(self):
         # type: () -> List[AnchorPoint]
         """
@@ -1560,12 +1796,12 @@ class NodeItem(QGraphicsWidget):
 
     def itemChange(self, change, value):
         # type: (QGraphicsItem.GraphicsItemChange, Any) -> Any
-        if change == QGraphicsItem.ItemSelectedChange:
+        if change == QGraphicsItem.ItemSelectedHasChanged:
             self.shapeItem.setSelected(value)
             self.captionTextItem.setSelectionState(value)
+            self.selectedChanged.emit(value)
         elif change == QGraphicsItem.ItemPositionHasChanged:
             self.positionChanged.emit()
-
         return super().itemChange(change, value)
 
     def __updatePalette(self):
@@ -1641,3 +1877,6 @@ def parse_format_fields(format_str):
                      for _, field, spec, conv in formatter.parse(format_str)
                      if field is not None]
     return format_fields
+
+
+from .linkitem import LinkItem
