@@ -55,6 +55,7 @@ from ..canvas.items.annotationitem import Annotation as AnnotationItem
 from . import interactions
 from . import commands
 from . import quickmenu
+from ..utils import findf
 
 Pos = Tuple[float, float]
 RuntimeState = signalmanager.SignalManager.State
@@ -181,6 +182,9 @@ class SchemeEditWidget(QWidget):
         # list of links when set to a clean state
         self.__cleanLinks = []
 
+        # list of annotations when set to a clean state
+        self.__cleanAnnotations = []
+
         self.__editFinishedMapper = QSignalMapper(self)
         self.__editFinishedMapper.mapped[QObject].connect(
             self.__onEditingFinished
@@ -452,11 +456,13 @@ class SchemeEditWidget(QWidget):
         )
         self.__saveWindowGroupAction = QAction(
             self.tr("Save Window Group..."), self,
+            objectName="window-groups-save-action",
             toolTip="Create and save a new window group."
         )
         self.__saveWindowGroupAction.triggered.connect(self.__saveWindowGroup)
         self.__clearWindowGroupsAction = QAction(
             self.tr("Delete All Groups"), self,
+            objectName="window-groups-clear-action",
             toolTip="Delete all saved widget presets"
         )
         self.__clearWindowGroupsAction.triggered.connect(
@@ -523,6 +529,7 @@ class SchemeEditWidget(QWidget):
         scene.focusItemChanged.connect(self.__onFocusItemChanged)
         scene.selectionChanged.connect(self.__onSelectionChanged)
         scene.link_item_activated.connect(self.__onLinkActivate)
+        scene.link_item_added.connect(self.__onLinkAdded)
         scene.node_item_activated.connect(self.__onNodeActivate)
         scene.annotation_added.connect(self.__onAnnotationAdded)
         scene.annotation_removed.connect(self.__onAnnotationRemoved)
@@ -605,14 +612,17 @@ class SchemeEditWidget(QWidget):
         if not modified:
             if self.__scheme:
                 self.__cleanProperties = node_properties(self.__scheme)
-                self.__cleanLinks = list(self.__scheme.links)
+                self.__cleanLinks = self.__scheme.links
+                self.__cleanAnnotations = self.__scheme.annotations
             else:
                 self.__cleanProperties = {}
                 self.__cleanLinks = []
+                self.__cleanAnnotations = []
             self.__undoStack.setClean()
         else:
             self.__cleanProperties = {}
             self.__cleanLinks = []
+            self.__cleanAnnotations = []
 
     modified = Property(bool, fget=isModified, fset=setModified)
 
@@ -640,23 +650,36 @@ class SchemeEditWidget(QWidget):
         Returns node properties differences since last clean state,
         excluding unclean nodes.
         """
+
         currentProperties = node_properties(self.__scheme)
+        # ignore diff for newly created nodes
+        cleanNodes = self.cleanNodes()
         currentCleanNodeProperties = {k: v
                                       for k, v in currentProperties.items()
-                                      if k in self.cleanNodes()}
+                                      if k in cleanNodes}
+
+        cleanProperties = self.__cleanProperties
+        # ignore diff for deleted nodes
+        currentNodes = self.__scheme.nodes
+        cleanCurrentNodeProperties = {k: v
+                                      for k, v in cleanProperties.items()
+                                      if k in currentNodes}
 
         # ignore contexts
         ignore = set((node, "context_settings")
                      for node in currentCleanNodeProperties.keys())
 
         return list(dictdiffer.diff(
-            self.__cleanProperties,
+            cleanCurrentNodeProperties,
             currentCleanNodeProperties,
             ignore=ignore
         ))
 
     def restoreProperties(self, dict_diff):
-        dictdiffer.patch(dict_diff, node_properties(self.__scheme), in_place=True)
+        ref_properties = {
+            node: node.properties for node in self.__scheme.nodes
+        }
+        dictdiffer.patch(dict_diff, ref_properties, in_place=True)
 
     def cleanNodes(self):
         return list(self.__cleanProperties.keys())
@@ -664,6 +687,9 @@ class SchemeEditWidget(QWidget):
     def cleanLinks(self):
         return self.__cleanLinks
 
+    def cleanAnnotations(self):
+        return self.__cleanAnnotations
+
     def setQuickMenuTriggers(self, triggers):
         # type: (int) -> None
         """
@@ -757,6 +783,9 @@ class SchemeEditWidget(QWidget):
         if self.__scheme is not scheme:
             if self.__scheme:
                 self.__scheme.title_changed.disconnect(self.titleChanged)
+                self.__scheme.window_group_presets_changed.disconnect(
+                    self.__reset_window_group_menu
+                )
                 self.__scheme.removeEventFilter(self)
                 sm = self.__scheme.findChild(signalmanager.SignalManager)
                 if sm:
@@ -778,8 +807,12 @@ class SchemeEditWidget(QWidget):
             if self.__scheme:
                 self.__scheme.title_changed.connect(self.titleChanged)
                 self.titleChanged.emit(scheme.title)
+                self.__scheme.window_group_presets_changed.connect(
+                    self.__reset_window_group_menu
+                )
                 self.__cleanProperties = node_properties(scheme)
-                self.__cleanLinks = list(scheme.links)
+                self.__cleanLinks = scheme.links
+                self.__cleanAnnotations = scheme.annotations
                 sm = scheme.findChild(signalmanager.SignalManager)
                 if sm:
                     sm.stateChanged.connect(self.__signalManagerStateChanged)
@@ -793,6 +826,7 @@ class SchemeEditWidget(QWidget):
             else:
                 self.__cleanProperties = {}
                 self.__cleanLinks = []
+                self.__cleanAnnotations = []
 
             self.__teardownScene(self.__scene)
             self.__scene.deleteLater()
@@ -811,27 +845,7 @@ class SchemeEditWidget(QWidget):
                 nodes = self.__scheme.nodes
                 if nodes:
                     self.ensureVisible(nodes[0])
-
-        group = self.__windowGroupsActionGroup
-        menu = self.__windowGroupsAction.menu()
-        actions = group.actions()
-        for a in actions:
-            group.removeAction(a)
-            menu.removeAction(a)
-            a.deleteLater()
-
-        if scheme:
-            presets = scheme.window_group_presets()
-            sep = menu.findChild(QAction, "groups-separator")
-            for g in presets:
-                a = QAction(g.name, menu)
-                a.setShortcut(
-                    QKeySequence("Meta+P, Ctrl+{}"
-                                 .format(len(group.actions()) + 1))
-                )
-                a.setData(g)
-                group.addAction(a)
-                menu.insertAction(sep, a)
+        self.__reset_window_group_menu()
 
     def ensureVisible(self, node):
         # type: (SchemeNode) -> None
@@ -988,7 +1002,7 @@ class SchemeEditWidget(QWidget):
         else:
             nodes = []
         if nodes:
-            x, y = nodes[-1].position  # type: ignore
+            x, y = nodes[-1].position
             position = (x + 150, y)
         else:
             position = (150, 150)
@@ -1044,21 +1058,21 @@ class SchemeEditWidget(QWidget):
             raise NoWorkflowError()
         source_node = old_link.source_node
         sink_node = old_link.sink_node
+        source_channel = old_link.source_channel
+        sink_channel = old_link.sink_channel
 
-        possible_links = (self.__scheme.propose_links(source_node, new_node),
+        proposed_links = (self.__scheme.propose_links(source_node, new_node),
                           self.__scheme.propose_links(new_node, sink_node))
-
-        first_link_sink_channel = [l[1] for l in possible_links[0]
-                                   if l[0] == old_link.source_channel][0]
-        second_link_source_channel = [l[0] for l in possible_links[1]
-                                      if l[1] == old_link.sink_channel][0]
-
+        # Preserve existing {source,sink}_channel if possible; use first
+        # proposed if not.
+        first = findf(proposed_links[0], lambda t: t[0] == source_channel,
+                      default=proposed_links[0][0])
+        second = findf(proposed_links[1], lambda t: t[1] == sink_channel,
+                       default=proposed_links[1][0])
         new_links = (
-            SchemeLink(source_node, old_link.source_channel,
-                       new_node, first_link_sink_channel),
-            SchemeLink(new_node, second_link_source_channel,
-                       sink_node, old_link.sink_channel))
-
+            SchemeLink(source_node, first[0], new_node, first[1]),
+            SchemeLink(new_node, second[0], sink_node, second[1])
+        )
         command = commands.InsertNodeCommand(self.__scheme, new_node, old_link, new_links)
         self.__undoStack.push(command)
 
@@ -1098,10 +1112,15 @@ class SchemeEditWidget(QWidget):
             return
         scene = self.scene()
         self.__undoStack.beginMacro(self.tr("Remove"))
+        # order LinkItem removes before NodeItems; Removing NodeItems also
+        # removes links so some links in selected could already be removed by
+        # a preceding NodeItem remove
+        selected = sorted(
+            selected, key=lambda item: not isinstance(item, items.LinkItem))
         for item in selected:
             assert self.__scheme is not None
             if isinstance(item, items.NodeItem):
-                node = self.scene().node_for_item(item)
+                node = scene.node_for_item(item)
                 self.__undoStack.push(
                     commands.RemoveNodeCommand(self.__scheme, node)
                 )
@@ -1109,10 +1128,15 @@ class SchemeEditWidget(QWidget):
                 if item.hasFocus() or item.isAncestorOf(scene.focusItem()):
                     # Clear input focus from the item to be removed.
                     scene.focusItem().clearFocus()
-                annot = self.scene().annotation_for_item(item)
+                annot = scene.annotation_for_item(item)
                 self.__undoStack.push(
                     commands.RemoveAnnotationCommand(self.__scheme, annot)
                 )
+            elif isinstance(item, items.LinkItem):
+                link = scene.link_for_item(item)
+                self.__undoStack.push(
+                    commands.RemoveLinkCommand(self.__scheme, link)
+                )
         self.__undoStack.endMacro()
 
     def selectAll(self):
@@ -1182,6 +1206,11 @@ class SchemeEditWidget(QWidget):
         return list(map(self.scene().node_for_item,
                         self.scene().selected_node_items()))
 
+    def selectedLinks(self):
+        # type: () -> List[SchemeLink]
+        return list(map(self.scene().link_for_item,
+                        self.scene().selected_link_items()))
+
     def selectedAnnotations(self):
         # type: () -> List[BaseSchemeAnnotation]
         """
@@ -1340,7 +1369,6 @@ class SchemeEditWidget(QWidget):
             self.removeLink(link)
             event.accept()
             return True
-
         any_item = scene.item_at(pos)
         if not any_item:
             self.__emptyClickButtons |= event.button()
@@ -1454,6 +1482,8 @@ class SchemeEditWidget(QWidget):
 
     def sceneKeyPressEvent(self, event):
         # type: (QKeyEvent) -> bool
+        self.__updateOpenWidgetAnchors(event)
+
         scene = self.__scene
         if scene.user_interaction_handler:
             return False
@@ -1504,8 +1534,18 @@ class SchemeEditWidget(QWidget):
 
     def sceneKeyReleaseEvent(self, event):
         # type: (QKeyEvent) -> bool
+        self.__updateOpenWidgetAnchors(event)
         return False
 
+    def __updateOpenWidgetAnchors(self, event=None):
+        scene = self.__scene
+        # Open widget anchors on shift. New link action should work during this
+        if event:
+            open = event.modifiers() == Qt.ShiftModifier
+        else:
+            open = QApplication.keyboardModifiers() == Qt.ShiftModifier
+        scene.set_widget_anchors_open(open)
+
     def sceneContextMenuEvent(self, event):
         # type: (QGraphicsSceneContextMenuEvent) -> bool
         scenePos = event.scenePos()
@@ -1582,15 +1622,17 @@ class SchemeEditWidget(QWidget):
         # type: () -> None
         self.sender().ended.disconnect(self.__onInteractionEnded)
         set_enabled_all(self.__disruptiveActions, True)
+        self.__updateOpenWidgetAnchors()
 
     def __onSelectionChanged(self):
         # type: () -> None
         nodes = self.selectedNodes()
         annotations = self.selectedAnnotations()
+        links = self.selectedLinks()
 
         self.__openSelectedAction.setEnabled(bool(nodes))
         self.__removeSelectedAction.setEnabled(
-            bool(nodes) or bool(annotations)
+            bool(nodes or annotations or links)
         )
 
         self.__helpAction.setEnabled(len(nodes) == 1)
@@ -1603,15 +1645,11 @@ class SchemeEditWidget(QWidget):
         else:
             self.__openSelectedAction.setText(self.tr("Open"))
 
-        if len(nodes) + len(annotations) > 1:
+        if len(nodes) + len(annotations) + len(links) > 1:
             self.__removeSelectedAction.setText(self.tr("Remove All"))
         else:
             self.__removeSelectedAction.setText(self.tr("Remove"))
 
-        if len(nodes) == 0:
-            self.__openSelectedAction.setText(self.tr("Open"))
-            self.__removeSelectedAction.setText(self.tr("Remove"))
-
         focus = self.focusNode()
         if focus is not None:
             desc = focus.description
@@ -1632,6 +1670,9 @@ class SchemeEditWidget(QWidget):
                                                   link.sink_node)
         action.edit_links()
 
+    def __onLinkAdded(self, item: items.LinkItem) -> None:
+        item.setFlag(QGraphicsItem.ItemIsSelectable)
+
     def __onNodeActivate(self, item):
         # type: (items.NodeItem) -> None
         node = self.__scene.node_for_item(item)
@@ -1900,7 +1941,7 @@ class SchemeEditWidget(QWidget):
 
         view = self.view()
         try:
-            action = menu.exec_(view.mapToGlobal(view.mapFromScene(QPoint(x, y))))
+            action = menu.exec_(view.mapToGlobal(view.mapFromScene(QPointF(x, y))))
         finally:
             menu.setFilterFunc(None)
 
@@ -2112,6 +2153,34 @@ class SchemeEditWidget(QWidget):
             role = QPalette.Window
         self.__view.viewport().setBackgroundRole(role)
 
+    def __reset_window_group_menu(self):
+        group = self.__windowGroupsActionGroup
+        menu = self.__windowGroupsAction.menu()
+        # remove old actions
+        actions = group.actions()
+        for a in actions:
+            group.removeAction(a)
+            menu.removeAction(a)
+            a.deleteLater()
+
+        sep = menu.findChild(QAction, "groups-separator")
+
+        workflow = self.__scheme
+        if workflow is None:
+            return
+
+        presets = workflow.window_group_presets()
+
+        for g in presets:
+            a = QAction(g.name, menu)
+            a.setShortcut(
+                QKeySequence("Meta+P, Ctrl+{}"
+                             .format(len(group.actions()) + 1))
+            )
+            a.setData(g)
+            group.addAction(a)
+            menu.insertAction(sep, a)
+
     def __saveWindowGroup(self):
         # type: () -> None
         """Run a 'Save Window Group' dialog"""
@@ -2130,13 +2199,9 @@ class SchemeEditWidget(QWidget):
         if default:
             dlg.setDefaultIndex(default[0])
 
-        menu = self.__windowGroupsAction.menu()  # type: QMenu
-        group = self.__windowGroupsActionGroup
-
         def store_group():
             text = dlg.selectedText()
             default = dlg.isDefaultChecked()
-            actions = group.actions()  # type: List[QAction]
             try:
                 idx = items.index(text)
             except ValueError:
@@ -2146,44 +2211,21 @@ class SchemeEditWidget(QWidget):
             if idx == -1:
                 # new group slot
                 newpresets.append(newpreset)
-                action = QAction(text, menu)
-                action.setShortcut(
-                    QKeySequence("Meta+P, Ctrl+{}".format(len(newpresets)))
-                )
-                oldpreset = None
             else:
                 newpresets[idx] = newpreset
 
-                action = actions[idx]
-                # store old state for undo
-                oldpreset = presets[idx]
-
             if newpreset.default:
                 idx_ = idx if idx >= 0 else len(newpresets) - 1
                 for g in newpresets[:idx_] + newpresets[idx_ + 1:]:
                     g.default = False
-            sep = menu.findChild(QAction, "groups-separator")
-            assert isinstance(sep, QAction) and sep.isSeparator()
 
-            def redo():
-                action.setData(newpreset)
-                workflow.set_window_group_presets(newpresets)
-                if idx == -1:
-                    group.addAction(action)
-                    menu.insertAction(sep, action)
-
-            def undo():
-                action.setData(oldpreset)
-                workflow.set_window_group_presets(presets)
-                if idx == -1:
-                    group.removeAction(action)
-                    menu.removeAction(action)
             if idx == -1:
                 text = "Store Window Group"
             else:
                 text = "Update Window Group"
+
             self.__undoStack.push(
-                commands.SimpleUndoCommand(redo, undo, text)
+                commands.SetWindowGroupPresets(workflow, newpresets, text=text)
             )
         dlg.accepted.connect(store_group)
         dlg.show()
@@ -2201,26 +2243,9 @@ class SchemeEditWidget(QWidget):
         workflow = self.__scheme
         if workflow is None:
             return
-        presets = workflow.window_group_presets()
-        menu = self.__windowGroupsAction.menu()  # type: QMenu
-        group = self.__windowGroupsActionGroup
-        actions = group.actions()
-
-        def redo():
-            workflow.set_window_group_presets([])
-            for action in reversed(actions):
-                group.removeAction(action)
-                menu.removeAction(action)
-
-        def undo():
-            workflow.set_window_group_presets(presets)
-            sep = menu.findChild(QAction, "groups-separator")
-            for action in actions:
-                group.addAction(action)
-                menu.insertAction(sep, action)
-
         self.__undoStack.push(
-            commands.SimpleUndoCommand(redo, undo, "Delete All Window Groups")
+            commands.SetWindowGroupPresets(
+                workflow, [], text="Delete All Window Groups")
         )
 
     def __raiseToFont(self):
@@ -2403,10 +2428,12 @@ def node_properties(scheme):
 
 def can_insert_node(new_node_desc, original_link):
     # type: (WidgetDescription, SchemeLink) -> bool
-    return any(scheme.compatible_channels(original_link.source_channel, input)
-               for input in new_node_desc.inputs) and \
-           any(scheme.compatible_channels(output, original_link.sink_channel)
-               for output in new_node_desc.outputs)
+    return any(any(scheme.compatible_channels(output, input)
+                   for input in new_node_desc.inputs)
+               for output in original_link.source_node.output_channels()) and \
+           any(any(scheme.compatible_channels(output, input)
+                   for output in new_node_desc.outputs)
+               for input in original_link.sink_node.input_channels())
 
 
 def uniquify(item, names, pattern="{item}-{_}", start=0):
