@@ -16,10 +16,8 @@ from AnyQt.QtGui import QKeySequence
 from AnyQt.QtWidgets import QWidget, QLabel, QAction
 
 from orangecanvas.resources import icon_loader
-from orangecanvas.scheme import (
-    SchemeNode, Scheme, NodeEvent, SchemeLink, LinkEvent
-)
-from orangecanvas.scheme.events import WorkflowEvent, WorkflowEnvChanged
+from orangecanvas.scheme import SchemeNode, Scheme, NodeEvent, LinkEvent
+from orangecanvas.scheme.events import WorkflowEvent
 from orangecanvas.scheme.node import UserMessage
 
 log = logging.getLogger(__name__)
@@ -53,21 +51,26 @@ class WidgetManager(QObject):
     The widgets created with :func:`create_widget_for_node` will automatically
     receive dispatched events:
 
-        * :data:`WorkflowEvent.InputLinkAdded` - when a new input link is
+        * :attr:`.WorkflowEvent.InputLinkAdded` - when a new input link is
           added to the workflow.
-        * :data:`LinkEvent.InputLinkRemoved` - when a input link is removed.
-        * :data:`LinkEvent.OutputLinkAdded` - when a new output link is
+        * :attr:`.WorkflowEvent.InputLinkRemoved` - when a input link is
+          removed.
+        * :attr:`.WorkflowEvent.OutputLinkAdded` - when a new output link is
           added to the workflow.
-        * :data:`LinkEvent.InputLinkRemoved` - when a output link is removed.
-        * :data:`LinkEvent.InputLinkStateChanged` - when the input link's
+        * :attr:`.WorkflowEvent.OutputLinkRemoved` - when a output link is
+          removed.
+        * :attr:`.WorkflowEvent.InputLinkStateChange` - when the input link's
           runtime state changes.
-        * :data:`LinkEvent.OutputLinkStateChanged` - when the output link's
+        * :attr:`.WorkflowEvent.OutputLinkStateChange` - when the output link's
           runtime state changes.
-        * :data:`WorkflowEnvEvent.WorkflowEnvironmentChanged` - when the
+        * :attr:`.WorkflowEvent.NodeStateChange` - when the node's runtime
+          state changes.
+        * :attr:`.WorkflowEvent.WorkflowEnvironmentChange` - when the
           workflow environment changes.
 
     .. seealso:: :func:`.Scheme.add_link()`, :func:`Scheme.remove_link`,
-                 :func:`.Scheme.runtime_env`
+                 :func:`.Scheme.runtime_env`, :class:`NodeEvent`,
+                 :class:`LinkEvent`
     """
     #: A new QWidget was created and added by the manager.
     widget_for_node_added = Signal(SchemeNode, QWidget)
@@ -98,7 +101,7 @@ class WidgetManager(QObject):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
         self.__workflow = None  # type: Optional[Scheme]
-        self.__creation_policy = WidgetManager.Normal
+        self.__creation_policy = WidgetManager.OnDemand
         self.__float_widgets_on_top = False
 
         self.__item_for_node = {}  # type: Dict[SchemeNode, Item]
@@ -127,9 +130,6 @@ class WidgetManager(QObject):
                 self.__remove_node(node)
             self.__workflow.node_added.disconnect(self.__on_node_added)
             self.__workflow.node_removed.disconnect(self.__on_node_removed)
-            self.__workflow.link_added.disconnect(self.__on_link_added)
-            self.__workflow.link_removed.disconnect(self.__on_link_removed)
-            self.__workflow.runtime_env_changed.disconnect(self.__on_env_changed)
             self.__workflow.removeEventFilter(self)
 
         self.__workflow = workflow
@@ -138,12 +138,6 @@ class WidgetManager(QObject):
             self.__on_node_added, Qt.UniqueConnection)
         workflow.node_removed.connect(
             self.__on_node_removed, Qt.UniqueConnection)
-        workflow.link_added.connect(
-            self.__on_link_added, Qt.UniqueConnection)
-        workflow.link_removed.connect(
-            self.__on_link_removed, Qt.UniqueConnection)
-        workflow.runtime_env_changed.connect(
-            self.__on_env_changed, Qt.UniqueConnection)
         workflow.installEventFilter(self)
         for node in workflow.nodes:
             self.__add_node(node)
@@ -302,12 +296,12 @@ class WidgetManager(QObject):
         workflow = self.__workflow
         assert workflow is not None
         inputs = workflow.find_links(sink_node=node)
-        for link in inputs:
-            ev = LinkEvent(LinkEvent.InputLinkAdded, link)
+        for i, link in enumerate(inputs):
+            ev = LinkEvent(LinkEvent.InputLinkAdded, link, i)
             QCoreApplication.sendEvent(w, ev)
         outputs = workflow.find_links(source_node=node)
-        for link in outputs:
-            ev = LinkEvent(LinkEvent.OutputLinkAdded, link)
+        for i, link in enumerate(outputs):
+            ev = LinkEvent(LinkEvent.OutputLinkAdded, link, i)
             QCoreApplication.sendEvent(w, ev)
 
         self.widget_for_node_added.emit(node, w)
@@ -380,34 +374,6 @@ class WidgetManager(QObject):
                 if self.__init_queue:
                     self.__init_timer.start()
 
-    def __on_link_added(self, link):  # type: (SchemeLink) -> None
-        assert self.__workflow is not None
-        assert link.source_node in self.__workflow.nodes
-        assert link.sink_node in self.__workflow.nodes
-        source = self.__item_for_node.get(link.source_node)
-        sink = self.__item_for_node.get(link.sink_node)
-        # notify the node gui of an added link
-        if source is not None and source.widget is not None:
-            ev = LinkEvent(LinkEvent.OutputLinkAdded, link)
-            QCoreApplication.sendEvent(source.widget, ev)
-        if sink is not None and sink.widget is not None:
-            ev = LinkEvent(LinkEvent.InputLinkAdded, link)
-            QCoreApplication.sendEvent(sink.widget, ev)
-
-    def __on_link_removed(self, link):  # type: (SchemeLink) -> None
-        assert self.__workflow is not None
-        assert link.source_node in self.__workflow.nodes
-        assert link.sink_node in self.__workflow.nodes
-        source = self.__item_for_node.get(link.source_node)
-        sink = self.__item_for_node.get(link.sink_node)
-        # notify the node gui of an removed link
-        if source is not None and source.widget is not None:
-            ev = LinkEvent(LinkEvent.OutputLinkRemoved, link)
-            QCoreApplication.sendEvent(source.widget, ev)
-        if sink is not None and sink.widget is not None:
-            ev = LinkEvent(LinkEvent.InputLinkRemoved, link)
-            QCoreApplication.sendEvent(sink.widget, ev)
-
     def __mark_activated(self, widget):  # type: (QWidget) ->  None
         # Update the tracked stacking order for `widget`
         item = self.__item_for_widget.get(widget)
@@ -598,6 +564,10 @@ class WidgetManager(QObject):
             if event.type() == NodeEvent.NodeActivateRequest:
                 self.__activate_widget_for_node(recv)
             self.__dispatch_events(recv, event)
+        elif event.type() == WorkflowEvent.WorkflowEnvironmentChange \
+                and recv is self.__workflow:
+            for node in self.__item_for_node:
+                self.__dispatch_events(node, event)
         return False
 
     def __dispatch_events(self, node: Node, event: QEvent) -> None:
@@ -639,13 +609,6 @@ class WidgetManager(QObject):
         if widget_was_visible:
             widget.show()
 
-    def __on_env_changed(self, key, newvalue, oldvalue):
-        # Notify widgets of a runtime environment change
-        for item in self.__item_for_node.values():
-            if item.widget is not None:
-                ev = WorkflowEnvChanged(key, newvalue, oldvalue)
-                QCoreApplication.sendEvent(item.widget, ev)
-
     def actions_for_context_menu(self, node):
         # type: (SchemeNode) -> List[QAction]
         """
